"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAsyncIteratorNextTask = getAsyncIteratorNextTask;
exports.reduceUntilWithIndexLimited = reduceUntilWithIndexLimited;
/**
 * @since 1.0.0
 */
const O = __importStar(require("fp-ts/lib/Option"));
const node_process_1 = require("node:process");
/**
 * @category sequencing
 * @since 1.0.0
 */
function getAsyncIteratorNextTask(iterable) {
    // let i = 0;
    async function next() {
        for await (const value of iterable) {
            return O.some(value);
        }
        return O.none;
    }
    return next;
}
/**
 * generic async iterable reducer.
 *
 * - limit: how many values pull in parallel.
 * - until: predicate if the function should keep pulling values from the iterator
 * - b: reducer accumulator initial value.
 * - f: reducer function.
 *
 * @category sequencing
 * @since 1.0.0
 */
function reduceUntilWithIndexLimited(limit, until, b, f) {
    return (iterable) => () => new Promise((resolve, reject) => {
        let running = 0;
        let out = b;
        let index = 0;
        let isDone = false;
        const next = getAsyncIteratorNextTask(iterable);
        function run() {
            if (isDone || // we done
                running === limit // the tasks queue is saturated
            ) {
                return;
            }
            running++;
            next()
                .then(didRun)
                .catch((cause) => {
                isDone = true;
                reject(Error("This should never have happened. Use AsyncIterableEither or AsyncIterableOption tryCatch function to handle errors", { cause }));
            });
        }
        function didRun(value) {
            if (O.isNone(value)) {
                isDone = true;
            }
            else {
                out = f(index++, out, value.value);
                if (!until(value.value)) {
                    isDone = true;
                }
            }
            running--;
            // keeps it safe from race conditions
            // due to some iterators resolve synchronously
            (0, node_process_1.nextTick)(() => {
                run();
                if (running === 0) {
                    resolve(out);
                }
            });
        }
        while (running < limit) {
            run();
        }
    });
}
