import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, } from "fp-ts/lib/Apply";
import * as A from "fp-ts/lib/Array";
import * as chainable from "fp-ts/lib/Chain";
import * as E from "fp-ts/lib/Either";
import * as ET from "fp-ts/lib/EitherT";
import { fromEitherK as fromEitherK_, fromOptionK as fromOptionK_, fromOption as fromOption_, fromPredicate as fromPredicate_, } from "fp-ts/lib/FromEither";
import { fromIOK as fromIOK_ } from "fp-ts/lib/FromIO";
import { fromTaskK as fromTaskK_ } from "fp-ts/lib/FromTask";
import { bindTo as bindTo_, flap as flap_, let as let_, } from "fp-ts/lib/Functor";
import * as O from "fp-ts/lib/Option";
import * as T from "fp-ts/lib/Task";
import * as TE from "fp-ts/lib/TaskEither";
import * as TO from "fp-ts/lib/TaskOption";
import { flow, identity, pipe } from "fp-ts/lib/function";
import * as AI from "./AsyncIterable";
import { reduceUntilWithIndexLimited } from "./AsyncIterableReduce";
import * as I from "./Iterable";
import { asUnit as asUnit_, as as as_, dual, tapIO as tapIO_, tapTask as tapTask_, tap as tap_, } from "./internal";
import { flatMap as flatMap_, mapBoth as mapBoth_, mapError as mapError_, tapEither as tapEither_, tapError as tapError_, } from "./internalEither";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export const URI = "AsyncIterableEither";
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 1.0.0
 */
export const left = ET.left(AI.Pointed);
/**
 * @category constructors
 * @since 1.0.0
 */
export const right = ET.right(AI.Pointed);
/**
 * @category constructors
 * @since 1.0.0
 */
export const rightIterable = (ma) => AI.fromIterable(pipe(ma, I.map(E.right)));
/**
 * @category constructors
 * @since 1.0.0
 */
export const leftIterable = (ma) => AI.fromIterable(pipe(ma, I.map(E.left)));
/**
 * @category constructors
 * @since 1.0.0
 */
export const rightTask = (ma) => AI.fromTask(pipe(ma, T.map(E.right)));
/**
 * @category constructors
 * @since 1.0.0
 */
export const leftTask = (ma) => AI.fromTask(pipe(ma, T.map(E.left)));
/**
 * @category constructors
 * @since 1.0.0
 */
export const of = right;
/**
 * @category constructors
 * @since 1.0.0
 */
export const rightAsyncIterable = ET.rightF(AI.Functor);
/**
 * @category constructors
 * @since 1.0.0
 */
export const leftAsyncIterable = ET.leftF(AI.Functor);
/**
 * @category constructors
 * @since 1.0.0
 */
export const rightIO = flow(AI.fromIO, rightAsyncIterable);
/**
 * @category constructors
 * @since 1.0.0
 */
export const leftIO = flow(AI.fromIO, leftAsyncIterable);
// -------------------------------------------------------------------------------------
// conversions
// -------------------------------------------------------------------------------------
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromIO = rightIO;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromTask = rightTask;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromTaskEither = AI.fromTask;
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromTaskEitherK = (f) => flow(f, fromTaskEither);
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromTaskOption = (onNone) => (fa) => AI.fromTask(pipe(fa, T.map(E.fromOption(onNone))));
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromIterable = rightIterable;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromAsyncIterable = rightAsyncIterable;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromEither = AI.of;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromIOEither = AI.fromIO;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromAsyncIterableOption = (onNone) => AI.map(E.fromOption(onNone));
/**
 * @category pattern matching
 * @since 1.0.0
 */
export const match = ET.match(AI.Functor);
/**
 * Less strict version of [`match`](#match).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const matchW = match;
/**
 * The `E` suffix (short for **E**ffect) means that the handlers return an AsyncIterable.
 *
 * @category pattern matching
 * @since 1.0.0
 */
export function matchE(onLeft, onRight) {
    return AI.flatMapTask(E.match(onLeft, onRight));
}
/**
 * Less strict version of [`matchE`](#matche).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const matchEW = matchE;
/**
 * @category error handling
 * @since 1.0.0
 */
export function getOrElse(onLeft) {
    return match(onLeft, identity);
    // return AI.map(E.match(onLeft, identity));
}
/**
 * Less strict version of [`getOrElse`](#getorelse).
 *
 * The `W` suffix (short for **W**idening) means that the handler return type will be merged.
 *
 * @category error handling
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const getOrElseW = 
/* eslint-disable-next-line @typescript-eslint/no-explicit-any */
getOrElse;
/**
 * @category interop
 * @since 1.0.0
 */
export function tryCatch(onRejected) {
    return (fa) => ({
        async *[Symbol.asyncIterator]() {
            try {
                for await (const a of fa) {
                    yield E.right(a);
                }
            }
            catch (e) {
                yield pipe(e, onRejected, E.left);
            }
        },
    });
}
/**
 * @category interop
 * @since 1.0.0
 * alias for tryCatch(E.toError)
 */
export function tryCatchToError() {
    return tryCatch(E.toError);
}
/**
 * @category conversions
 * @since 1.0.0
 */
export const toUnion = ET.toUnion(AI.Functor);
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromNullable = ET.fromNullable(AI.Pointed);
/**
 * @since 1.0.0
 */
export const swap = ET.swap(AI.Functor);
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category mapping
 * @since 1.0.0
 */
export const map = ET.map(AI.Functor);
/**
 * Returns a `AsyncIterableEither` whose failure and success channels have been mapped by the specified pair of functions, `f` and `g`.
 *
 * @category error handling
 * @since 1.0.0
 */
export const mapBoth = dual(3, mapBoth_(AI.Functor));
/**
 * Returns a `AsyncIterableEither` with its error channel mapped using the specified function.
 *
 * @category error handling
 * @since 1.0.0
 */
export const mapError = dual(2, mapError_(AI.Functor));
/**
 * @category apply
 * @since 1.0.0
 */
export const ap = ET.ap(AI.Apply);
/**
 * Less strict version of [`ap`](#ap).
 *
 * The `W` suffix (short for **W**idening) means that the error types will be merged.
 *
 * @category apply
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const apW = ap;
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMap = dual(2, flatMap_(AI.Monad));
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapIterable = (f) => (ma) => pipe(ma, AI.flatMapIterable((e) => E.isLeft(e) ? I.of(e) : pipe(e.right, f, I.map((E.right)))));
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapEither = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield pipe(a, E.flatMap(f));
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapOption = (f, onNone) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield pipe(a, E.flatMapOption(f, onNone));
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapTask = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield await pipe(a, TE.fromEither, TE.flatMapTask(f))();
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapTaskEither = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield await pipe(a, TE.fromEither, TE.flatMap(f))();
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapTaskOption = (f, onNone) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield await pipe(a, TO.fromEither, TO.flatMap(f), TE.fromTaskOption(onNone))();
        }
    },
});
/**
 * @category instances
 * @since 1.0.0
 */
export const Pointed = { URI, of };
/**
 * @category instances
 * @since 1.0.0
 */
export const Functor = {
    URI,
    map: (fa, f) => map(f)(fa),
};
/**
 * Maps the value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export const as = dual(2, as_(Functor));
/**
 * Maps the value to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export const asUnit = asUnit_(Functor);
/**
 * @category mapping
 * @since 1.0.0
 */
export const flap = flap_(Functor);
/**
 * @category instances
 * @since 1.0.0
 */
export const Apply = {
    ...Functor,
    ap: (fab, fa) => ap(fa)(fab),
};
/**
 * @category apply
 * @since 1.0.0
 */
export const apFirst = apFirst_(Apply);
/**
 * @category apply
 * @since 1.0.0
 */
export const apSecond = apSecond_(Apply);
/**
 * @category instances
 * @since 1.0.0
 */
export const Applicative = {
    ...Pointed,
    ...Apply,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const Chain = {
    ...Apply,
    chain: (fab, fa) => flatMap(fa)(fab),
};
/**
 * @category instances
 * @since 1.0.0
 */
export const FromIO = {
    URI,
    fromIO,
};
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromIOK = fromIOK_(FromIO);
/**
 * @category instances
 * @since 1.0.0
 */
export const FromTask = {
    URI,
    fromIO,
    fromTask,
};
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromTaskK = fromTaskK_(FromTask);
/**
 * @category instances
 * @since 1.0.0
 */
export const FromEither = {
    URI,
    fromEither,
};
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromEitherK = fromEitherK_(FromEither);
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromOption = fromOption_(FromEither);
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromOptionK = fromOptionK_(FromEither);
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromPredicate = fromPredicate_(FromEither);
/**

/**
 * @category instances
 * @since 1.0.0
 */
export const Monad = {
    ...Pointed,
    ...Chain,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const throwError = left;
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadThrow = {
    ...Monad,
    throwError,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadIO = {
    ...Monad,
    fromIO,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadTask = {
    ...MonadIO,
    fromTask,
};
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export const filterWithIndex = (predicateWithIndex) => AI.filterWithIndex((i, a) => {
    if (E.isLeft(a)) {
        return true;
    }
    return predicateWithIndex(i, a.right);
});
/**
 * @category filtering
 * @since 1.0.0
 */
export const filter = (predicate) => filterWithIndex((_, a) => predicate(a));
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterTaskWithIndex = (predicateWithIndex) => AI.filterTaskWithIndex((i, a) => {
    if (E.isLeft(a)) {
        return T.of(true);
    }
    return predicateWithIndex(i, a.right);
});
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterTask = (predicate) => filterTaskWithIndex((_, a) => predicate(a));
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterTaskEitherWithIndex = (predicateWithIndex) => (fa) => {
    let i = 0;
    async function* next() {
        for await (const a of fa) {
            if (E.isLeft(a)) {
                yield a;
            }
            else {
                const predicateEither = await predicateWithIndex(i++, a.right)();
                if (E.isLeft(predicateEither)) {
                    yield predicateEither;
                }
                else if (predicateEither.right) {
                    yield a;
                }
                else {
                    yield* next();
                }
            }
        }
    }
    return AI.fromAsyncGenerator(next);
};
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterTaskEither = (predicate) => filterTaskEitherWithIndex((_, a) => predicate(a));
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export const filterMapWithIndex = (f) => (fa) => {
    let i = 0;
    async function* next() {
        for await (const a of fa) {
            if (E.isLeft(a)) {
                yield a;
            }
            else {
                const optionB = f(i++, a.right);
                if (O.isSome(optionB)) {
                    yield E.right(optionB.value);
                }
                else {
                    yield* next();
                }
            }
        }
    }
    return AI.fromAsyncGenerator(next);
};
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMap = (f) => filterMapWithIndex((_, a) => f(a));
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMapTaskWithIndex = (f) => (fa) => {
    let i = 0;
    async function* next() {
        for await (const a of fa) {
            if (E.isLeft(a)) {
                yield a;
            }
            else {
                const optionB = await f(i++, a.right)();
                if (O.isSome(optionB)) {
                    yield E.right(optionB.value);
                }
                else {
                    yield* next();
                }
            }
        }
    }
    return AI.fromAsyncGenerator(next);
};
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMapTask = (f) => filterMapTaskWithIndex((_, a) => f(a));
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMapTaskEitherWithIndex = (f) => (fa) => {
    let i = 0;
    async function* next() {
        for await (const a of fa) {
            if (E.isLeft(a)) {
                yield a;
            }
            else {
                const eitherOptionB = await f(i++, a.right)();
                if (E.isLeft(eitherOptionB)) {
                    yield eitherOptionB;
                }
                else if (O.isSome(eitherOptionB.right)) {
                    yield E.right(eitherOptionB.right.value);
                }
                else {
                    yield* next();
                }
            }
        }
    }
    return AI.fromAsyncGenerator(next);
};
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMapTaskEither = (f) => filterMapTaskEitherWithIndex((_, a) => f(a));
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export const tap = dual(2, tap_(Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapTask = dual(2, tapTask_(FromTask, Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapTaskEither = /*#__PURE__*/ dual(2, (self, f) => tap(self, fromTaskEitherK(f)));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapIO = dual(2, tapIO_(FromIO, Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapEither = dual(2, tapEither_(FromEither, Chain));
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category error handling
 * @since 1.0.0
 */
export const tapError = dual(2, tapError_(AI.Monad));
/**
 * @category error handling
 * @since 1.0.0
 */
export const tapErrorIO = (onLeft) => tapError(fromIOK(onLeft));
/**
 * @category error handling
 * @since 1.0.0
 */
export const tapErrorTask = (onLeft) => tapError(fromTaskK(onLeft));
/**
 * @category error handling
 * @since 1.0.0
 */
export const tapErrorTaskEither = (onLeft) => tapError(fromTaskEitherK(onLeft));
/**
 * Less strict version of [`flatten`](#flatten).
 *
 * The `W` suffix (short for **W**idening) means that the error types will be merged.
 *
 * @category sequencing
 * @since 1.0.0
 */
export const flattenW = flatMap(identity);
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatten = flattenW;
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flattenIterable = flatMapIterable(identity);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @category do notation
 * @since 1.0.0
 */
export const Do = of({});
/**
 * @category do notation
 * @since 1.0.0
 */
export const bind = chainable.bind(Chain);
/**
 * @category do notation
 * @since 1.0.0
 */
export const bindTo = bindTo_(Functor);
const _let = let_(Functor);
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export const apS = apS_(Apply);
/**
 * the order of the results does not corresponds to the order of async iterator elements
 * @category folding
 * @since 1.0.0
 */
export function toIterableLimited(limit) {
    return flow(reduceUntilWithIndexLimited(limit, E.isRight, E.right(A.zero()), (_, b, a) => {
        if (E.isLeft(a)) {
            return a;
        }
        if (E.isLeft(b)) {
            return b;
        }
        b.right.push(a.right);
        return b;
    }), TE.map(I.fromReadonlyArray));
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toIterablePar(limit) {
    return toIterableLimited(limit);
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toIterableSeq() {
    return toIterableLimited(1);
}
/**
 * preserves the order of elements coming from async iterator and corresponding results
 * @category folding
 * @since 1.0.0
 */
export function toArrayLimited(limit) {
    return reduceUntilWithIndexLimited(limit, E.isRight, E.right(A.zero()), (i, b, a) => {
        if (E.isLeft(a)) {
            return a;
        }
        if (E.isLeft(b)) {
            return b;
        }
        b.right[i] = a.right;
        return b;
    });
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toArrayPar(limit) {
    return toArrayLimited(limit);
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toArraySeq() {
    return toArrayLimited(1);
}
