import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, } from "fp-ts/lib/Apply";
import * as A from "fp-ts/lib/Array";
import * as chainable from "fp-ts/lib/Chain";
import { compact as compact_ } from "fp-ts/lib/Compactable";
import { filterMap as filterMap_, filter as filter_, } from "fp-ts/lib/Filterable";
import { fromEitherK as fromEitherK_ } from "fp-ts/lib/FromEither";
import { fromIOK as fromIOK_ } from "fp-ts/lib/FromIO";
import { fromTaskK as fromTaskK_ } from "fp-ts/lib/FromTask";
import { bindTo as bindTo_, flap as flap_, let as let_, } from "fp-ts/lib/Functor";
import * as O from "fp-ts/lib/Option";
import * as OT from "fp-ts/lib/OptionT";
import * as T from "fp-ts/lib/Task";
import * as TO from "fp-ts/lib/TaskOption";
import { flow, identity, pipe } from "fp-ts/lib/function";
import * as AI from "./AsyncIterable";
import { reduceUntilWithIndexLimited } from "./AsyncIterableReduce";
import * as I from "./Iterable";
import { asUnit as asUnit_, as as as_, dual, tapIO as tapIO_, tapTask as tapTask_, tap as tap_, } from "./internal";
import { tapEither as tapEither_ } from "./internalEither";
import { flatMap as flatMap_ } from "./internalOption";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export const URI = "AsyncIterableOption";
/**
 * @category constructors
 * @since 1.0.0
 */
export const some = OT.some(AI.Pointed);
/**
 * @category constructors
 * @since 1.0.0
 */
export const of = some;
/**
 * @category constructors
 * @since 1.0.0
 */
export const zero = () => AI.of(O.none);
/**
 * @category constructors
 * @since 1.0.0
 */
export const none = /*#__PURE__*/ zero;
/**
 * @category constructors
 * @since 1.0.0
 */
export const someIterable = (fa) => AI.fromIterable(pipe(fa, I.map(O.some)));
/**
 * @category constructors
 * @since 1.0.0
 */
export const someAsyncIterable = AI.map(O.some);
/**
 * @category constructors
 * @since 1.0.0
 */
export const someTask = (a) => AI.fromTask(pipe(a, T.map(O.some)));
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromPredicate = OT.fromPredicate(AI.Pointed);
// -------------------------------------------------------------------------------------
// conversions
// -------------------------------------------------------------------------------------
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromEither = OT.fromEither(AI.Pointed);
/**
 * @category instances
 * @since 1.0.0
 */
export const FromEither = {
    URI,
    fromEither,
};
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromOption = AI.of;
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromOptionK = /*#__PURE__*/ OT.fromOptionK(AI.Pointed);
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromEitherK = 
/*#__PURE__*/ fromEitherK_(FromEither);
/**
 * @category constructors
 * @since 1.0.0
 */
export const someIO = (fa) => AI.fromIO(() => pipe(fa(), O.some));
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromIO = someIO;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromIterable = someIterable;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromAsyncIterable = someAsyncIterable;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromAsyncIterableEither = (fa) => pipe(fa, AI.map(O.fromEither));
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromTask = someTask;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromTaskEither = (fa) => AI.fromTask(pipe(fa, T.map(O.fromEither)));
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromTaskEitherK = (f) => flow(f, fromTaskEither);
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromTaskOption = (fa) => AI.fromTask(fa);
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromTaskOptionK = (f) => flow(f, fromTaskOption);
/**
 * @category pattern matching
 * @since 1.0.0
 */
export const match = OT.match(AI.Functor);
/**
 * Less strict version of [`match`](#match).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const matchW = match;
/**
 * The `E` suffix (short for **E**ffect) means that the handlers return an effect (`Task`).
 *
 * @category pattern matching
 * @since 1.0.0
 */
export function matchE(onNone, onSome) {
    return AI.flatMapTask(O.match(onNone, onSome));
}
/**
 * Less strict version of [`matchE`](#matche).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const matchEW = matchE;
/**
 * @category error handling
 * @since 1.0.0
 */
export function getOrElse(onNone) {
    return AI.map(O.getOrElse(onNone));
}
/**
 * Less strict version of [`getOrElse`](#getorelse).
 *
 * The `W` suffix (short for **W**idening) means that the handler return type will be merged.
 *
 * @category error handling
 * @since 1.0.0
 */
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment */
export const getOrElseW = getOrElse;
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromNullable = OT.fromNullable(AI.Pointed);
/**
 * @category interop
 * @since 1.0.0
 */
export function tryCatch(fa) {
    return {
        async *[Symbol.asyncIterator]() {
            try {
                for await (const a of fa) {
                    yield O.some(a);
                }
            }
            catch (_e) {
                yield O.none;
            }
        },
    };
}
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category mapping
 * @since 1.0.0
 */
export const map = OT.map(AI.Functor);
/**
 * @since 1.0.0
 * @category apply
 */
export const ap = OT.ap(AI.Apply);
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMap = dual(2, flatMap_(AI.Monad));
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapIterable = (f) => (ma) => pipe(ma, AI.flatMapIterable((o) => O.isNone(o) ? I.of(o) : pipe(o.value, f, I.map(O.some))));
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapEither = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield pipe(a, O.flatMap(flow(f, O.fromEither)));
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapOption = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield pipe(a, O.flatMap(f));
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapTask = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield await pipe(a, TO.fromOption, TO.flatMapTask(f))();
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapTaskEither = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield await pipe(a, TO.fromOption, TO.flatMap(flow(f, T.map(O.fromEither))))();
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMapTaskOption = (f) => (fa) => ({
    async *[Symbol.asyncIterator]() {
        for await (const a of fa) {
            yield await pipe(a, TO.fromOption, TO.flatMap(f))();
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatten = flatMap(identity);
/**
 * @category filtering
 * @since 1.0.0
 */
export const compact = compact_(AI.Functor, O.Compactable);
/**
 * @category filtering
 * @since 1.0.0
 */
export const filter = filter_(AI.Functor, O.Filterable);
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMap = filterMap_(AI.Functor, O.Filterable);
/**
 * @category instances
 * @since 1.0.0
 */
export const Pointed = { URI, of };
/**
 * @category instances
 * @since 1.0.0
 */
export const Functor = {
    URI,
    map: (fa, f) => pipe(fa, map(f)),
};
/**
 * Maps the value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export const as = dual(2, as_(Functor));
/**
 * Maps the `Some` value of this `TaskOption` to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export const asUnit = asUnit_(Functor);
/**
 * @category mapping
 * @since 1.0.0
 */
export const flap = /*#__PURE__*/ flap_(Functor);
/**
 * @category instances
 * @since 1.0.0
 */
export const Apply = {
    ...Functor,
    ap: (fab, fa) => ap(fa)(fab),
};
/**
 * @category apply
 * @since 1.0.0
 */
export const apFirst = apFirst_(Apply);
/**
 * @category apply
 * @since 1.0.0
 */
export const apSecond = apSecond_(Apply);
/**
 * @category instances
 * @since 1.0.0
 */
export const Applicative = {
    ...Pointed,
    ...Apply,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const Chain = {
    ...Apply,
    chain: (fab, fa) => flatMap(fa)(fab),
};
/**
 * @category instances
 * @since 1.0.0
 */
export const FromIO = {
    URI,
    fromIO,
};
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromIOK = /*#__PURE__*/ fromIOK_(FromIO);
/**
 * @category instances
 * @since 1.0.0
 */
export const FromTask = {
    URI,
    fromIO,
    fromTask,
};
/**
 * @category lifting
 * @since 1.0.0
 */
export const fromTaskK = /*#__PURE__*/ fromTaskK_(FromTask);
/**
 * @category instances
 * @since 1.0.0
 */
export const Monad = {
    ...Pointed,
    ...Chain,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const throwError = none;
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadThrow = {
    ...Monad,
    throwError,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadIO = {
    ...Monad,
    fromIO,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadTask = {
    ...MonadIO,
    fromTask,
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export const tap = /*#__PURE__*/ dual(2, tap_(Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapEither = /*#__PURE__*/ dual(2, tapEither_(FromEither, Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapIO = /*#__PURE__*/ dual(2, tapIO_(FromIO, Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapTask = /*#__PURE__*/ dual(2, tapTask_(FromTask, Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapTaskEither = /*#__PURE__*/ dual(2, (self, f) => tap(self, fromTaskEitherK(f)));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapTaskOption = /*#__PURE__*/ dual(2, (self, f) => tap(self, fromTaskOptionK(f)));
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @category do notation
 * @since 1.0.0
 */
export const Do = /*#__PURE__*/ of({});
/**
 * @category do notation
 * @since 1.0.0
 */
export const bind = chainable.bind(Chain);
/**
 * @category do notation
 * @since 1.0.0
 */
export const bindTo = bindTo_(Functor);
const _let = let_(Functor);
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export const apS = /*#__PURE__*/ apS_(Apply);
/**
 * returns compacted Iterable
 * the order of the results does not corresponds to the order of async iterator elements
 * @category folding
 * @since 1.0.0
 */
export function toIterableLimited(limit) {
    return flow(toArrayLimited(limit), T.map(I.fromReadonlyArray));
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toIterablePar(limit) {
    return toIterableLimited(limit);
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toIterableSeq() {
    return toIterableLimited(1);
}
/**
 *  returns compacted Array
 *  the order of the results does not corresponds to the order of async iterator elements
 *  if you need to keep the order use AsyncIterator version of this function
 *  @category folding
 *  @since 1.0.0
 */
export function toArrayLimited(limit) {
    return reduceUntilWithIndexLimited(limit, O.isSome, A.zero(), (_, b, a) => {
        if (O.isNone(a)) {
            return b;
        }
        b.push(a.value);
        return b;
    });
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toArrayPar(limit) {
    return toArrayLimited(limit);
}
/**
 * @category folding
 * @since 1.0.0
 */
export function toArraySeq() {
    return toArrayLimited(1);
}
