/**
 * ```ts
 * export interface AsyncIterableEither<E, A> extends AsyncIterable<Either<E, A>> {}
 * ```
 *
 * `AsyncIterableEither<E, A>` represents an asynchronous stream that yields either of type `A` or yields an
 * error of type `E`. If you want to represent an asynchronous stream that never fails, please see `AsyncIterable`.
 *
 * @since 1.0.0
 */
import { Applicative2 } from "fp-ts/lib/Applicative";
import { Apply2 } from "fp-ts/lib/Apply";
import * as chainable from "fp-ts/lib/Chain";
import * as E from "fp-ts/lib/Either";
import { Either } from "fp-ts/lib/Either";
import { PredicateWithIndex, RefinementWithIndex } from "fp-ts/lib/FilterableWithIndex";
import { FromEither2 } from "fp-ts/lib/FromEither";
import { FromIO2 } from "fp-ts/lib/FromIO";
import { FromTask2 } from "fp-ts/lib/FromTask";
import { Functor2 } from "fp-ts/lib/Functor";
import { IO } from "fp-ts/lib/IO";
import { IOEither } from "fp-ts/lib/IOEither";
import { Monad2 } from "fp-ts/lib/Monad";
import { MonadIO2 } from "fp-ts/lib/MonadIO";
import { MonadThrow2 } from "fp-ts/lib/MonadThrow";
import * as O from "fp-ts/lib/Option";
import { Pointed2 } from "fp-ts/lib/Pointed";
import { Predicate } from "fp-ts/lib/Predicate";
import { Refinement } from "fp-ts/lib/Refinement";
import * as T from "fp-ts/lib/Task";
import * as TE from "fp-ts/lib/TaskEither";
import * as TO from "fp-ts/lib/TaskOption";
import { LazyArg } from "fp-ts/lib/function";
import { MonadTask2 } from "fp-ts/lib/MonadTask";
import * as AI from "./AsyncIterable";
import { AsyncIterableOption } from "./AsyncIterableOption";
/**
 * @category model
 * @since 1.0.0
 */
export interface AsyncIterableEither<E, A> extends AsyncIterable<Either<E, A>> {
}
/**
 * @category type lambdas
 * @since 1.0.0
 */
export declare const URI = "AsyncIterableEither";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export type URI = typeof URI;
declare module "fp-ts/lib/HKT" {
    interface URItoKind2<E, A> {
        readonly [URI]: AsyncIterableEither<E, A>;
    }
}
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const left: <E = never, A = never>(e: E) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const right: <E = never, A = never>(a: A) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const rightIterable: <E = never, A = never>(ma: Iterable<A>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const leftIterable: <E = never, A = never>(ma: Iterable<E>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const rightTask: <E = never, A = never>(ma: T.Task<A>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const leftTask: <E = never, A = never>(me: T.Task<E>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const of: <E = never, A = never>(a: A) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const rightAsyncIterable: <E = never, A = never>(ma: AsyncIterable<A>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const leftAsyncIterable: <E = never, A = never>(me: AsyncIterable<E>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const rightIO: <E = never, A = never>(ma: IO<A>) => AsyncIterableEither<E, A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const leftIO: <E = never, A = never>(me: IO<E>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIO: <A, E = never>(fa: IO<A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTask: <E, A>(fa: T.Task<A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTaskEither: <E, A>(fa: TE.TaskEither<E, A>) => AsyncIterableEither<E, A>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromTaskEitherK: <E, A extends ReadonlyArray<unknown>, B>(f: (...a: A) => TE.TaskEither<E, B>) => ((...a: A) => AsyncIterableEither<E, B>);
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTaskOption: <E, A>(onNone: LazyArg<E>) => (fa: TO.TaskOption<A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIterable: <A, E = never>(fa: Iterable<A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromAsyncIterable: <A, E = never>(fa: AsyncIterable<A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromEither: <E, A>(fa: Either<E, A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIOEither: <E, A>(fa: IOEither<E, A>) => AsyncIterableEither<E, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromAsyncIterableOption: <E>(onNone: LazyArg<E>) => <A>(fa: AsyncIterableOption<A>) => AsyncIterableEither<E, A>;
/**
 * @category pattern matching
 * @since 1.0.0
 */
export declare const match: <E, B, A>(onLeft: (e: E) => B, onRight: (a: A) => B) => (ma: AsyncIterableEither<E, A>) => AsyncIterable<B>;
/**
 * Less strict version of [`match`](#match).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
export declare const matchW: <E, B, A, C>(onLeft: (e: E) => B, onRight: (a: A) => C) => (ma: AsyncIterableEither<E, A>) => AsyncIterable<B | C>;
/**
 * The `E` suffix (short for **E**ffect) means that the handlers return an AsyncIterable.
 *
 * @category pattern matching
 * @since 1.0.0
 */
export declare function matchE<E, A, B>(onLeft: (e: E) => T.Task<B>, onRight: (a: A) => T.Task<B>): (fa: AsyncIterable<E.Either<E, A>>) => AsyncIterable<B>;
/**
 * Less strict version of [`matchE`](#matche).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
export declare const matchEW: <E, B, A, C>(onLeft: (e: E) => T.Task<B>, onRight: (a: A) => T.Task<C>) => (ma: AsyncIterableEither<E, A>) => AsyncIterable<B | C>;
/**
 * @category error handling
 * @since 1.0.0
 */
export declare function getOrElse<E, A>(onLeft: (e: E) => A): (ma: AsyncIterableEither<E, A>) => AsyncIterable<A>;
/**
 * Less strict version of [`getOrElse`](#getorelse).
 *
 * The `W` suffix (short for **W**idening) means that the handler return type will be merged.
 *
 * @category error handling
 * @since 1.0.0
 */
export declare const getOrElseW: <E, B>(onLeft: (e: E) => B) => <A>(ma: AsyncIterableEither<E, A>) => AsyncIterable<A | B>;
/**
 * @category interop
 * @since 1.0.0
 */
export declare function tryCatch<E, A>(onRejected: (cause: unknown) => E): (fa: AsyncIterable<A>) => AsyncIterableEither<E, A>;
/**
 * @category interop
 * @since 1.0.0
 * alias for tryCatch(E.toError)
 */
export declare function tryCatchToError<A>(): (fa: AsyncIterable<A>) => AsyncIterableEither<Error, A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const toUnion: <E, A>(fa: AsyncIterableEither<E, A>) => AsyncIterable<E | A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromNullable: <E>(e: E) => <A>(a: A) => AsyncIterableEither<E, NonNullable<A>>;
/**
 * @since 1.0.0
 */
export declare const swap: <E, A>(ma: AsyncIterableEither<E, A>) => AsyncIterableEither<A, E>;
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <E>(fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * Returns a `AsyncIterableEither` whose failure and success channels have been mapped by the specified pair of functions, `f` and `g`.
 *
 * @category error handling
 * @since 1.0.0
 */
export declare const mapBoth: {
    <E, G, A, B>(f: (e: E) => G, g: (a: A) => B): (self: AsyncIterableEither<E, A>) => AsyncIterableEither<G, B>;
    <E, A, G, B>(self: AsyncIterableEither<E, A>, f: (e: E) => G, g: (a: A) => B): AsyncIterableEither<G, B>;
};
/**
 * Returns a `AsyncIterableEither` with its error channel mapped using the specified function.
 *
 * @category error handling
 * @since 1.0.0
 */
export declare const mapError: {
    <E, G>(f: (e: E) => G): <A>(self: AsyncIterableEither<E, A>) => AsyncIterableEither<G, A>;
    <E, A, G>(self: AsyncIterableEither<E, A>, f: (e: E) => G): AsyncIterableEither<G, A>;
};
/**
 * @category apply
 * @since 1.0.0
 */
export declare const ap: <E, A>(fa: AsyncIterableEither<E, A>) => <B>(fab: AsyncIterableEither<E, (a: A) => B>) => AsyncIterableEither<E, B>;
/**
 * Less strict version of [`ap`](#ap).
 *
 * The `W` suffix (short for **W**idening) means that the error types will be merged.
 *
 * @category apply
 * @since 1.0.0
 */
export declare const apW: <E2, A>(fa: AsyncIterableEither<E2, A>) => <E1, B>(fab: AsyncIterableEither<E1, (a: A) => B>) => AsyncIterableEither<E1 | E2, B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMap: {
    <A, E2, B>(f: (a: A) => AsyncIterableEither<E2, B>): <E1>(ma: AsyncIterableEither<E1, A>) => AsyncIterableEither<E1 | E2, B>;
    <E1, A, E2, B>(ma: AsyncIterableEither<E1, A>, f: (a: A) => AsyncIterableEither<E2, B>): AsyncIterableEither<E1 | E2, B>;
};
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapIterable: <E, A, B>(f: (a: A) => Iterable<B>) => (ma: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapEither: <E1, E2, A, B>(f: (a: A) => E.Either<E2, B>) => (fa: AsyncIterableEither<E1, A>) => AsyncIterableEither<E1 | E2, B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapOption: <E, A, B>(f: (a: A) => O.Option<B>, onNone: LazyArg<E>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTask: <E, A, B>(f: (a: A) => T.Task<B>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTaskEither: <E1, E2, A, B>(f: (a: A) => TE.TaskEither<E2, B>) => (fa: AsyncIterableEither<E1, A>) => AsyncIterableEither<E1 | E2, B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTaskOption: <E, A, B>(f: (a: A) => TO.TaskOption<B>, onNone: LazyArg<E>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Pointed: Pointed2<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Functor: Functor2<URI>;
/**
 * Maps the value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const as: {
    <A>(a: A): <E, _>(self: AsyncIterableEither<E, _>) => AsyncIterableEither<E, A>;
    <E, _, A>(self: AsyncIterableEither<E, _>, a: A): AsyncIterableEither<E, A>;
};
/**
 * Maps the value to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const asUnit: <E, _>(self: AsyncIterableEither<E, _>) => AsyncIterableEither<E, void>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: AsyncIterableEither<E, (a: A) => B>) => AsyncIterableEither<E, B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Apply: Apply2<URI>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apFirst: <E, B>(second: AsyncIterableEither<E, B>) => <A>(first: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apSecond: <E, B>(second: AsyncIterableEither<E, B>) => <A>(first: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Applicative: Applicative2<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Chain: chainable.Chain2<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromIO: FromIO2<URI>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => IO<B>) => <E = never>(...a: A) => AsyncIterableEither<E, B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromTask: FromTask2<URI>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromTaskK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => T.Task<B>) => <E = never>(...a: A) => AsyncIterableEither<E, B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromEither: FromEither2<URI>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromEitherK: <E, A extends ReadonlyArray<unknown>, B>(f: (...a: A) => E.Either<E, B>) => (...a: A) => AsyncIterableEither<E, B>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromOption: <E>(onNone: LazyArg<E>) => <A>(fa: O.Option<A>) => AsyncIterableEither<E, A>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromOptionK: <E>(onNone: LazyArg<E>) => <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => O.Option<B>) => (...a: A) => AsyncIterableEither<E, B>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromPredicate: {
    <E, A, B extends A>(refinement: Refinement<A, B>, onFalse: (a: A) => E): (a: A) => AsyncIterableEither<E, B>;
    <E, A>(predicate: Predicate<A>, onFalse: (a: A) => E): <B extends A>(b: B) => AsyncIterableEither<E, B>;
    <E, A>(predicate: Predicate<A>, onFalse: (a: A) => E): (a: A) => AsyncIterableEither<E, A>;
};
/**

/**
 * @category instances
 * @since 1.0.0
 */
export declare const Monad: Monad2<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const throwError: MonadThrow2<URI>["throwError"];
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadThrow: MonadThrow2<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadIO: MonadIO2<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadTask: MonadTask2<URI>;
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export declare const filterWithIndex: {
    <E, A, B extends A>(refinementWithIndex: RefinementWithIndex<number, A, B>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
    <E, A>(predicateWithIndex: PredicateWithIndex<number, A>): <B extends A>(fb: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
    <E, A>(predicateWithIndex: PredicateWithIndex<number, A>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filter: {
    <E, A, B extends A>(refinement: Refinement<A, B>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
    <E, A>(predicate: Predicate<A>): <B extends A>(fb: AsyncIterableEither<E, B>) => AsyncIterableEither<E, B>;
    <E, A>(predicate: Predicate<A>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterTaskWithIndex: {
    <E, A>(predicateWithIndex: AI.PredicateTaskWithIndex<number, A>): <B extends A>(fb: AsyncIterableEither<E, B>) => AsyncIterableEither<E, B>;
    <E, A>(predicateWithIndex: AI.PredicateTaskWithIndex<number, A>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterTask: {
    <E, A>(predicate: AI.PredicateTask<A>): <B extends A>(fb: AsyncIterableEither<E, B>) => AsyncIterableEither<E, B>;
    <E, A>(predicate: AI.PredicateTask<A>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export interface PredicateTaskEitherWithIndex<I, E, A> {
    (i: I, a: A): TE.TaskEither<E, boolean>;
}
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterTaskEitherWithIndex: {
    <E, A>(predicateWithIndex: PredicateTaskEitherWithIndex<number, E, A>): <B extends A>(fb: AsyncIterableEither<E, B>) => AsyncIterableEither<E, B>;
    <E, A>(predicateWithIndex: PredicateTaskEitherWithIndex<number, E, A>): (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export interface PredicateTaskEither<E, A> {
    (a: A): TE.TaskEither<E, boolean>;
}
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterTaskEither: {
    <E, A>(predicate: PredicateTaskEither<E, A>): <B extends A>(fb: AsyncIterableEither<E, B>) => AsyncIterableEither<E, B>;
    <E, A>(predicate: PredicateTaskEither<E, A>): (fa: AsyncIterableEither<E, A>) => AsyncIterable<A>;
};
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapWithIndex: <E, A, B>(f: (i: number, a: A) => O.Option<B>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMap: <E, A, B>(f: (a: A) => O.Option<B>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapTaskWithIndex: <E, A, B>(f: (i: number, a: A) => T.Task<O.Option<B>>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapTask: <E, A, B>(f: (a: A) => T.Task<O.Option<B>>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapTaskEitherWithIndex: <E, A, B>(f: (i: number, a: A) => TE.TaskEither<E, O.Option<B>>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapTaskEither: <E, A, B>(f: (a: A) => TE.TaskEither<E, O.Option<B>>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, B>;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export declare const tap: {
    <E1, A, E2, _>(self: AsyncIterableEither<E1, A>, f: (a: A) => AsyncIterableEither<E2, _>): AsyncIterableEither<E1 | E2, A>;
    <A, E2, _>(f: (a: A) => AsyncIterableEither<E2, _>): <E1>(self: AsyncIterableEither<E1, A>) => AsyncIterableEither<E2 | E1, A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapTask: {
    <A, _>(f: (a: A) => T.Task<_>): <E>(self: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
    <E, A, _>(self: AsyncIterableEither<E, A>, f: (a: A) => T.Task<_>): AsyncIterableEither<E, A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapTaskEither: {
    <A, E2, _>(f: (a: A) => TE.TaskEither<E2, _>): <E1>(self: AsyncIterableEither<E1, A>) => AsyncIterableEither<E1 | E2, A>;
    <E1, A, E2, _>(self: AsyncIterableEither<E1, A>, f: (a: A) => TE.TaskEither<E2, _>): AsyncIterableEither<E1 | E2, A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapIO: {
    <A, _>(f: (a: A) => IO<_>): <E>(self: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
    <E, A, _>(self: AsyncIterableEither<E, A>, f: (a: A) => IO<_>): AsyncIterableEither<E, A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapEither: {
    <A, E2, _>(f: (a: A) => Either<E2, _>): <E1>(self: AsyncIterableEither<E1, A>) => AsyncIterableEither<E2 | E1, A>;
    <E1, A, E2, _>(self: AsyncIterableEither<E1, A>, f: (a: A) => Either<E2, _>): AsyncIterableEither<E1 | E2, A>;
};
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category error handling
 * @since 1.0.0
 */
export declare const tapError: {
    <E1, E2, _>(onLeft: (e: E1) => AsyncIterableEither<E2, _>): <A>(self: AsyncIterableEither<E1, A>) => AsyncIterableEither<E1 | E2, A>;
    <E1, A, E2, _>(self: AsyncIterableEither<E1, A>, onLeft: (e: E1) => AsyncIterableEither<E2, _>): AsyncIterableEither<E1 | E2, A>;
};
/**
 * @category error handling
 * @since 1.0.0
 */
export declare const tapErrorIO: <E, B>(onLeft: (e: E) => IO<B>) => <A>(ma: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
/**
 * @category error handling
 * @since 1.0.0
 */
export declare const tapErrorTask: <E, B>(onLeft: (e: E) => T.Task<B>) => <A>(ma: AsyncIterableEither<E, A>) => AsyncIterableEither<E, A>;
/**
 * @category error handling
 * @since 1.0.0
 */
export declare const tapErrorTaskEither: <E1, E2, B>(onLeft: (e: E1) => TE.TaskEither<E2, B>) => <A>(ma: AsyncIterableEither<E1, A>) => AsyncIterableEither<E1 | E2, A>;
/**
 * Less strict version of [`flatten`](#flatten).
 *
 * The `W` suffix (short for **W**idening) means that the error types will be merged.
 *
 * @category sequencing
 * @since 1.0.0
 */
export declare const flattenW: <E1, E2, A>(mma: AsyncIterableEither<E1, AsyncIterableEither<E2, A>>) => AsyncIterableEither<E1 | E2, A>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatten: <E, A>(mma: AsyncIterableEither<E, AsyncIterableEither<E, A>>) => AsyncIterableEither<E, A>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flattenIterable: <E, A>(mma: AsyncIterableEither<E, Iterable<A>>) => AsyncIterableEither<E, A>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const Do: AsyncIterableEither<never, {}>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bind: <N extends string, A, E, B>(name: Exclude<N, keyof A>, f: (a: A) => AsyncIterableEither<E, B>) => (ma: AsyncIterableEither<E, A>) => AsyncIterableEither<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bindTo: <N extends string>(name: N) => <E, A>(fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, { readonly [K in N]: A; }>;
declare const _let: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => B) => <E>(fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const apS: <N extends string, A, E, B>(name: Exclude<N, keyof A>, fb: AsyncIterableEither<E, B>) => (fa: AsyncIterableEither<E, A>) => AsyncIterableEither<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * the order of the results does not corresponds to the order of async iterator elements
 * @category folding
 * @since 1.0.0
 */
export declare function toIterableLimited<E, A>(limit: number): (iterable: AsyncIterable<E.Either<E, A>>) => TE.TaskEither<E, Iterable<A>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toIterablePar<E, A>(limit: number): (iterable: AsyncIterable<E.Either<E, A>>) => TE.TaskEither<E, Iterable<A>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toIterableSeq<E, A>(): (iterable: AsyncIterable<E.Either<E, A>>) => TE.TaskEither<E, Iterable<A>>;
/**
 * preserves the order of elements coming from async iterator and corresponding results
 * @category folding
 * @since 1.0.0
 */
export declare function toArrayLimited<E, A>(limit: number): (iterable: AsyncIterable<E.Either<E, A>>) => T.Task<E.Either<E, A[]>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toArrayPar<E, A>(limit: number): (iterable: AsyncIterable<E.Either<E, A>>) => T.Task<E.Either<E, A[]>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toArraySeq<E, A>(): (iterable: AsyncIterable<E.Either<E, A>>) => T.Task<E.Either<E, A[]>>;
