/**
 * ```ts
 * export interface AsyncIterableOption<A> extends AsyncIterable<Option<A>> {}
 * ```
 *
 * `AsyncIterableOption<A>` represents an asynchronous stream that yields and optional values of type `A`.
 * If you want to represent an asynchronous stream that never fails, please see `AsyncIterable`.
 *
 * @since 1.0.0
 */
import { Applicative1 } from "fp-ts/lib/Applicative";
import { Apply1 } from "fp-ts/lib/Apply";
import * as chainable from "fp-ts/lib/Chain";
import { Compactable1 } from "fp-ts/lib/Compactable";
import { Either } from "fp-ts/lib/Either";
import { FromEither1 } from "fp-ts/lib/FromEither";
import { FromIO1 } from "fp-ts/lib/FromIO";
import { FromTask1 } from "fp-ts/lib/FromTask";
import { Functor1 } from "fp-ts/lib/Functor";
import { IO } from "fp-ts/lib/IO";
import { Monad1 } from "fp-ts/lib/Monad";
import { MonadIO1 } from "fp-ts/lib/MonadIO";
import { MonadTask1 } from "fp-ts/lib/MonadTask";
import { MonadThrow1 } from "fp-ts/lib/MonadThrow";
import * as O from "fp-ts/lib/Option";
import { Option } from "fp-ts/lib/Option";
import { Pointed1 } from "fp-ts/lib/Pointed";
import { Predicate } from "fp-ts/lib/Predicate";
import { Refinement } from "fp-ts/lib/Refinement";
import * as T from "fp-ts/lib/Task";
import { Task } from "fp-ts/lib/Task";
import * as TE from "fp-ts/lib/TaskEither";
import { TaskEither } from "fp-ts/lib/TaskEither";
import * as TO from "fp-ts/lib/TaskOption";
import { LazyArg } from "fp-ts/lib/function";
import { AsyncIterableEither } from "./AsyncIterableEither";
import { TaskOption } from "fp-ts/lib/TaskOption";
/**
 * @category model
 * @since 1.0.0
 */
export interface AsyncIterableOption<A> extends AsyncIterable<Option<A>> {
}
/**
 * @category type lambdas
 * @since 1.0.0
 */
export declare const URI = "AsyncIterableOption";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export type URI = typeof URI;
declare module "fp-ts/lib/HKT" {
    interface URItoKind<A> {
        readonly [URI]: AsyncIterableOption<A>;
    }
}
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const some: <A>(a: A) => AsyncIterableOption<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const of: <A>(a: A) => AsyncIterableOption<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const zero: <A>() => AsyncIterableOption<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const none: () => AsyncIterableOption<never>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const someIterable: <A>(fa: Iterable<A>) => AsyncIterableOption<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const someAsyncIterable: <A>(fa: AsyncIterable<A>) => AsyncIterableOption<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const someTask: <A>(a: Task<A>) => AsyncIterableOption<A>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromPredicate: {
    <A, B extends A>(refinement: Refinement<A, B>): (a: A) => AsyncIterableOption<B>;
    <A>(predicate: Predicate<A>): <B extends A>(b: B) => AsyncIterableOption<B>;
    <A>(predicate: Predicate<A>): (a: A) => AsyncIterableOption<A>;
};
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromEither: <A>(fa: Either<unknown, A>) => AsyncIterableOption<A>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromEither: FromEither1<URI>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromOption: <A>(fa: Option<A>) => AsyncIterableOption<A>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => Option<B>) => (...a: A) => AsyncIterableOption<B>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromEitherK: <E, A extends ReadonlyArray<unknown>, B>(f: (...a: A) => Either<E, B>) => (...a: A) => AsyncIterableOption<B>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const someIO: <A>(fa: IO<A>) => AsyncIterableOption<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIO: <A>(fa: IO<A>) => AsyncIterableOption<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIterable: <A>(fa: Iterable<A>) => AsyncIterableOption<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromAsyncIterable: <A>(fa: AsyncIterable<A>) => AsyncIterableOption<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromAsyncIterableEither: <E, A>(fa: AsyncIterableEither<E, A>) => AsyncIterableOption<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTask: <A>(fa: Task<A>) => AsyncIterableOption<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTaskEither: <_, A>(fa: TaskEither<_, A>) => AsyncIterableOption<A>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromTaskEitherK: <E, A extends ReadonlyArray<unknown>, B>(f: (...a: A) => TaskEither<E, B>) => ((...a: A) => AsyncIterableOption<B>);
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTaskOption: <A>(fa: TO.TaskOption<A>) => AsyncIterableOption<A>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromTaskOptionK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => TaskOption<B>) => ((...a: A) => AsyncIterableOption<B>);
/**
 * @category pattern matching
 * @since 1.0.0
 */
export declare const match: <B, A>(onNone: () => B, onSome: (a: A) => B) => (ma: AsyncIterableOption<A>) => AsyncIterable<B>;
/**
 * Less strict version of [`match`](#match).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
export declare const matchW: <B, A, C>(onNone: () => B, onSome: (a: A) => C) => (ma: AsyncIterableOption<A>) => AsyncIterable<B | C>;
/**
 * The `E` suffix (short for **E**ffect) means that the handlers return an effect (`Task`).
 *
 * @category pattern matching
 * @since 1.0.0
 */
export declare function matchE<A, B>(onNone: () => T.Task<B>, onSome: (a: A) => Task<B>): (fa: AsyncIterable<O.Option<A>>) => AsyncIterable<B>;
/**
 * Less strict version of [`matchE`](#matche).
 *
 * The `W` suffix (short for **W**idening) means that the handler return types will be merged.
 *
 * @category pattern matching
 * @since 1.0.0
 */
export declare const matchEW: <B, C, A>(onNone: () => Task<B>, onSome: (a: A) => Task<C>) => (fa: AsyncIterableOption<A>) => AsyncIterable<B | C>;
/**
 * @category error handling
 * @since 1.0.0
 */
export declare function getOrElse<A>(onNone: LazyArg<A>): (fa: AsyncIterableOption<A>) => AsyncIterable<A>;
/**
 * Less strict version of [`getOrElse`](#getorelse).
 *
 * The `W` suffix (short for **W**idening) means that the handler return type will be merged.
 *
 * @category error handling
 * @since 1.0.0
 */
export declare const getOrElseW: <B>(onNone: LazyArg<B>) => <A>(ma: AsyncIterableOption<A>) => AsyncIterable<A | B>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromNullable: <A>(a: A) => AsyncIterableOption<NonNullable<A>>;
/**
 * @category interop
 * @since 1.0.0
 */
export declare function tryCatch<A>(fa: AsyncIterable<A>): AsyncIterableOption<A>;
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @since 1.0.0
 * @category apply
 */
export declare const ap: <A>(fa: AsyncIterableOption<A>) => <B>(fab: AsyncIterableOption<(a: A) => B>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMap: {
    <A, B>(f: (a: A) => AsyncIterableOption<B>): (ma: AsyncIterableOption<A>) => AsyncIterableOption<B>;
    <A, B>(ma: AsyncIterableOption<A>, f: (a: A) => AsyncIterableOption<B>): AsyncIterableOption<B>;
};
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapIterable: <A, B>(f: (a: A) => Iterable<B>) => (ma: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapEither: <E, A, B>(f: (a: A) => Either<E, B>) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapOption: <A, B>(f: (a: A) => Option<B>) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTask: <A, B>(f: (a: A) => T.Task<B>) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTaskEither: <E, A, B>(f: (a: A) => TE.TaskEither<E, B>) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTaskOption: <A, B>(f: (a: A) => TO.TaskOption<B>) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatten: <A>(mma: AsyncIterableOption<AsyncIterableOption<A>>) => AsyncIterableOption<A>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const compact: Compactable1<URI>["compact"];
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filter: {
    <A, B extends A>(refinement: Refinement<A, B>): (fb: AsyncIterableOption<A>) => AsyncIterableOption<B>;
    <A>(predicate: Predicate<A>): <B extends A>(fb: AsyncIterableOption<B>) => AsyncIterableOption<B>;
    <A>(predicate: Predicate<A>): (fa: AsyncIterableOption<A>) => AsyncIterableOption<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => Option<B>) => (fga: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Pointed: Pointed1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Functor: Functor1<URI>;
/**
 * Maps the value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const as: {
    <A>(a: A): <_>(self: AsyncIterableOption<_>) => AsyncIterableOption<A>;
    <_, A>(self: AsyncIterableOption<_>, a: A): AsyncIterableOption<A>;
};
/**
 * Maps the `Some` value of this `TaskOption` to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const asUnit: <_>(self: AsyncIterableOption<_>) => AsyncIterableOption<void>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: AsyncIterableOption<(a: A) => B>) => AsyncIterableOption<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Apply: Apply1<URI>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apFirst: <B>(second: AsyncIterableOption<B>) => <A>(first: AsyncIterableOption<A>) => AsyncIterableOption<A>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apSecond: <B>(second: AsyncIterableOption<B>) => <A>(first: AsyncIterableOption<A>) => AsyncIterableOption<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Applicative: Applicative1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Chain: chainable.Chain1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromIO: FromIO1<URI>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => IO<B>) => (...a: A) => AsyncIterableOption<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromTask: FromTask1<URI>;
/**
 * @category lifting
 * @since 1.0.0
 */
export declare const fromTaskK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => T.Task<B>) => (...a: A) => AsyncIterableOption<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Monad: Monad1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const throwError: MonadThrow1<URI>["throwError"];
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadThrow: MonadThrow1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadIO: MonadIO1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadTask: MonadTask1<URI>;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export declare const tap: {
    <A, _>(self: AsyncIterableOption<A>, f: (a: A) => AsyncIterableOption<_>): AsyncIterableOption<A>;
    <A, _>(f: (a: A) => AsyncIterableOption<_>): (self: AsyncIterableOption<A>) => AsyncIterableOption<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapEither: {
    <A, E, _>(f: (a: A) => Either<E, _>): (self: AsyncIterableOption<A>) => AsyncIterableOption<A>;
    <A, E, _>(self: AsyncIterableOption<A>, f: (a: A) => Either<E, _>): AsyncIterableOption<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapIO: {
    <A, _>(f: (a: A) => IO<_>): (self: AsyncIterableOption<A>) => AsyncIterableOption<A>;
    <A, _>(self: AsyncIterableOption<A>, f: (a: A) => IO<_>): AsyncIterableOption<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapTask: {
    <A, _>(f: (a: A) => Task<_>): (self: AsyncIterableOption<A>) => AsyncIterableOption<A>;
    <A, _>(self: AsyncIterableOption<A>, f: (a: A) => Task<_>): AsyncIterableOption<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapTaskEither: {
    <A, _E, _B>(f: (a: A) => TaskEither<_E, _B>): (self: AsyncIterableOption<A>) => AsyncIterableOption<A>;
    <A, _E, _B>(self: AsyncIterableOption<A>, f: (a: A) => TaskEither<_E, _B>): AsyncIterableOption<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapTaskOption: {
    <A, _>(f: (a: A) => TaskOption<_>): (self: AsyncIterableOption<A>) => AsyncIterableOption<A>;
    <A, _>(self: AsyncIterableOption<A>, f: (a: A) => TaskOption<_>): AsyncIterableOption<A>;
};
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const Do: AsyncIterableOption<{}>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bind: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => AsyncIterableOption<B>) => (ma: AsyncIterableOption<A>) => AsyncIterableOption<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bindTo: <N extends string>(name: N) => <A>(fa: AsyncIterableOption<A>) => AsyncIterableOption<{ readonly [K in N]: A; }>;
declare const _let: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => B) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const apS: <N extends string, A, B>(name: Exclude<N, keyof A>, fb: AsyncIterableOption<B>) => (fa: AsyncIterableOption<A>) => AsyncIterableOption<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * returns compacted Iterable
 * the order of the results does not corresponds to the order of async iterator elements
 * @category folding
 * @since 1.0.0
 */
export declare function toIterableLimited<A>(limit: number): (iterable: AsyncIterable<O.Option<A>>) => T.Task<Iterable<A>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toIterablePar<A>(limit: number): (iterable: AsyncIterable<O.Option<A>>) => T.Task<Iterable<A>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toIterableSeq<A>(): (iterable: AsyncIterable<O.Option<A>>) => T.Task<Iterable<A>>;
/**
 *  returns compacted Array
 *  the order of the results does not corresponds to the order of async iterator elements
 *  if you need to keep the order use AsyncIterator version of this function
 *  @category folding
 *  @since 1.0.0
 */
export declare function toArrayLimited<A>(limit: number): (iterable: AsyncIterable<O.Option<A>>) => T.Task<A[]>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toArrayPar<A>(limit: number): (iterable: AsyncIterable<O.Option<A>>) => T.Task<A[]>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toArraySeq<A>(): (iterable: AsyncIterable<O.Option<A>>) => T.Task<A[]>;
